@@ -21,11 +21,12 @@
 
 */
 
+#include "cdda-cddb.h"
 
+#include <config.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <ctype.h>
 #include <sys/ioctl.h>
 #include <sys/types.h>
 #include <sys/socket.h>
@@ -38,12 +39,15 @@
 #include <errno.h>
 #include <unistd.h>
 
+#include <glib/gmessages.h>
+#include <glib/gstrfuncs.h>
+#include <glib/gutils.h>
+
 #define size16 short
 #define size32 int
 #include <cdda_interface.h>
 
 #include "cdda-cdrom-extensions.h"
-#include "cdda-cddb.h"
 
 /* This is here to work around a broken header file.
  * cdda_interface.h has a statically defined array of
@@ -68,6 +72,23 @@ static char *cddb_genres[] = {"unknown","blues","classical","country",
 			      "data","folk","jazz","misc","newage",
 			      "reggae","rock","soundtrack"};
 
+#ifdef ENABLE_IPV6
+/*Check whether the node is IPv6 enabled.*/
+static gboolean
+have_ipv6 (void)
+{
+	int s;
+
+	s = socket (AF_INET6, SOCK_STREAM, 0);
+	if (s != -1) {
+		close (s);
+		return TRUE;
+	}
+
+	return FALSE;
+}
+#endif
+
 /* CDDB sum function */
 static int 
 CDDBSum(int val)
@@ -131,10 +152,30 @@ static int
 CDDBConnect (CDDBServer *server)
 {
 	int sock;
+#ifdef ENABLE_IPV6
+	struct sockaddr_in6 sin6;
+	struct addrinfo hints, *result, *res;  /*info abt the IP of node*/
+#endif
 	struct sockaddr_in sin;
 	struct hostent *host;
 	char *sname;
   
+#ifdef ENABLE_IPV6
+	if (have_ipv6 ()) {
+		result = NULL;
+
+		memset (&sin6, 0 , sizeof (sin6));	
+		sin6.sin6_family = AF_INET6;
+
+		if (server->use_proxy) {
+			sin6.sin6_port = htons (server->proxy->port);
+		} else {
+			sin6.sin6_port = htons (server->port);
+		}
+	}
+#endif
+
+	memset (&sin, 0, sizeof (sin));
 	sin.sin_family = AF_INET;
 
 	if (server->use_proxy)
@@ -147,21 +188,81 @@ CDDBConnect (CDDBServer *server)
 	else
 		sname=server->name;
   
+#ifdef ENABLE_IPV6
+	if (have_ipv6 ()) {
+		memset (&hints, 0, sizeof (hints));
+		hints.ai_socktype = SOCK_STREAM;
+
+		if ((getaddrinfo (sname, NULL, &hints, &result)) != 0) {
+			return -1;
+		}
+      
+		for (res = result; res; res = res->ai_next) {
+
+			if (res->ai_family != AF_INET && res->ai_family != AF_INET6) {
+				continue;
+			}
+
+			sock = socket (res->ai_family, SOCK_STREAM, 0);
+			if (sock < 0) {
+				continue;
+			}
+
+			if (res->ai_family == AF_INET) {
+				memcpy (&sin.sin_addr, &((struct sockaddr_in *)res->ai_addr)->sin_addr, sizeof (struct in_addr));
+
+				if (connect (sock, (struct sockaddr *)&sin, sizeof (sin)) != -1) {
+					break;
+				}
+			}
+
+			if (res->ai_family == AF_INET6) {
+				memcpy (&sin6.sin6_addr, &((struct sockaddr_in6 *)res->ai_addr)->sin6_addr, sizeof (struct in6_addr));
+        
+				if (connect (sock, (struct sockaddr *)&sin6, sizeof (sin6)) != -1) {
+					break;
+				}
+			}
+
+			close (sock);
+		}
+
+		freeaddrinfo (result);
+
+		if (!res) {
+			/* No valid address found. */
+			return -1;
+		}
+	} else
+#endif  /*IPv4*/
+	{
+		sin.sin_addr.s_addr = inet_addr (sname);
 #ifdef SOLARIS
-  if((sin.sin_addr.s_addr = inet_addr(sname)) == (unsigned long)-1) {
+		if (sin.sin_addr.s_addr == (unsigned long)-1)
 #else
-  if((sin.sin_addr.s_addr = inet_addr(sname))==INADDR_NONE) {
+		if (sin.sin_addr.s_addr == INADDR_NONE)
 #endif
-    if((host = gethostbyname(sname))==NULL) return -1;
+		{
+			host = gethostbyname (sname);
+			if (host == NULL) {
+				return -1;
+			}
 
-    bcopy(host->h_addr,&sin.sin_addr,host->h_length);
-  }
+			bcopy (host->h_addr, &sin.sin_addr, host->h_length);
+		}
 
-  if((sock=socket(AF_INET,SOCK_STREAM,0))<0) return -1;
+		sock = socket (AF_INET, SOCK_STREAM, 0);
+		if (sock < 0) {
+			return -1;
+		}
   
-  if(connect(sock,(struct sockaddr *)&sin,sizeof(sin))<0) return -1;
-  
-  return sock;
+		if (connect (sock, (struct sockaddr *)&sin, sizeof (sin)) < 0) {
+			return -1;
+		}
+
+	}
+
+	return sock;
 }
 
 
@@ -375,11 +476,11 @@ char *ChopWhite(char *buf)
 {
   int pos;
 
-  for(pos=strlen(buf)-1;(pos>=0)&&isspace(buf[pos]);pos--);
+  for(pos=strlen(buf)-1;(pos>=0)&&g_ascii_isspace(buf[pos]);pos--);
 
   buf[pos+1]='\0';
 
-  for(;isspace(*buf);buf++);
+  for(;g_ascii_isspace(*buf);buf++);
 
   return buf;
 }
@@ -412,12 +513,12 @@ static void CDDBProcessLine(char *inbuffer,DiscData *data,
   int len = 0;
   char *st;
   
-  if(!strncasecmp(inbuffer,"DTITLE",6)) {
+  if(!g_ascii_strncasecmp(inbuffer,"DTITLE",6)) {
     len = strlen(data->data_title);
 
     strncpy(data->data_title+len,ChopWhite(inbuffer+7),256-len);
   }
-  else if(!strncasecmp(inbuffer,"DYEAR",5)) {
+  else if(!g_ascii_strncasecmp(inbuffer,"DYEAR",5)) {
     strtok(inbuffer,"=");
     
     st = strtok(NULL, "");
@@ -426,7 +527,7 @@ static void CDDBProcessLine(char *inbuffer,DiscData *data,
 
     data->data_year=atoi(ChopWhite(st));
   }
-  else if(!strncasecmp(inbuffer,"DGENRE",6)) {
+  else if(!g_ascii_strncasecmp(inbuffer,"DGENRE",6)) {
     strtok(inbuffer,"=");
     
     st = strtok(NULL, "");
@@ -435,7 +536,7 @@ static void CDDBProcessLine(char *inbuffer,DiscData *data,
     
     data->data_genre=CDDBGenreValue(ChopWhite(st));
   }
-  else if(!strncasecmp(inbuffer,"TTITLE",6)) {
+  else if(!g_ascii_strncasecmp(inbuffer,"TTITLE",6)) {
     track=atoi(strtok(inbuffer+6,"="));
     
     if(track<numtracks)
@@ -444,7 +545,7 @@ static void CDDBProcessLine(char *inbuffer,DiscData *data,
     strncpy(data->data_track[track].track_name+len,
 	    ChopWhite(strtok(NULL,"")),256-len);
   }
-  else if(!strncasecmp(inbuffer,"TARTIST",7)) {
+  else if(!g_ascii_strncasecmp(inbuffer,"TARTIST",7)) {
     data->data_multi_artist=TRUE;
 
     track=atoi(strtok(inbuffer+7,"="));
@@ -459,12 +560,12 @@ static void CDDBProcessLine(char *inbuffer,DiscData *data,
     strncpy(data->data_track[track].track_artist+len,
 	    ChopWhite(st),256-len);
   }
-  else if(!strncasecmp(inbuffer,"EXTD",4)) {
+  else if(!g_ascii_strncasecmp(inbuffer,"EXTD",4)) {
     len=strlen(data->data_extended);
 
     strncpy(data->data_extended+len,ChopWhite(inbuffer+5),4096-len);
   }
-  else if(!strncasecmp(inbuffer,"EXTT",4)) {
+  else if(!g_ascii_strncasecmp(inbuffer,"EXTT",4)) {
     track=atoi(strtok(inbuffer+4,"="));
     
     if(track<numtracks)
@@ -477,7 +578,7 @@ static void CDDBProcessLine(char *inbuffer,DiscData *data,
     strncpy(data->data_track[track].track_extended+len,
 	    ChopWhite(st),4096-len);
   }
-  else if(!strncasecmp(inbuffer,"PLAYORDER",5)) {
+  else if(!g_ascii_strncasecmp(inbuffer,"PLAYORDER",5)) {
     len=strlen(data->data_playlist);
 
     strncpy(data->data_playlist+len,ChopWhite(inbuffer+10),256-len);
