@@ -97,7 +97,7 @@ GET_PATH_MAX (void)
 #define OPEN open
 #endif
 
-#ifdef HAVE_LSEEK64
+#if defined(HAVE_LSEEK64) && defined(HAVE_OFF64_T)
 #define LSEEK lseek64
 #define OFF_T off64_t
 #else
@@ -628,6 +628,8 @@ get_stat_info (GnomeVFSFileInfo *file_info,
 	gboolean recursive;
 	char *link_file_path;
 	char *symlink_name;
+	char *symlink_dir;
+	char *newpath;
 	
 	followed_symlink = FALSE;
 	
@@ -685,6 +687,14 @@ get_stat_info (GnomeVFSFileInfo *file_info,
 				g_free (link_file_path);
 				return gnome_vfs_result_from_errno ();
 			}
+			if (symlink_name[0] != '/') {
+				symlink_dir = g_path_get_dirname (link_file_path);
+				newpath = g_build_filename (symlink_dir,
+							    symlink_name, NULL);
+				g_free (symlink_dir);
+				g_free (symlink_name);
+				symlink_name = newpath;
+			}
 			
 			if ((options & GNOME_VFS_FILE_INFO_FOLLOW_LINKS) == 0
 			                /* if we had an earlier ELOOP, don't get in an infinite loop here */
@@ -806,7 +816,9 @@ do_read_directory (GnomeVFSMethod *method,
 	result = readdir (handle->dir);
 
 	if (result == NULL && errno != 0) {
-		return gnome_vfs_result_from_errno ();
+		GnomeVFSResult ret = gnome_vfs_result_from_errno ();
+		G_UNLOCK (readdir);
+		return ret;
 	}
 	if (result != NULL) {
 		memcpy (handle->current_entry, result, sizeof (struct dirent));
@@ -945,9 +957,7 @@ do_is_local (GnomeVFSMethod *method,
 		struct stat statbuf;
 		if (stat (path, &statbuf) == 0) {
 			char *type = filesystem_type (path, path, &statbuf);
-			gboolean is_local = ((strcmp (type, "nfs") != 0) && 
-					     (strcmp (type, "afs") != 0) &&
-					     (strcmp (type, "ncpfs") != 0));
+			gboolean is_local = strcmp (type, "nfs") && strcmp (type, "afs");
 			local = GINT_TO_POINTER (is_local ? 1 : -1);
 			g_hash_table_insert (fstype_hash, path, local);
 		}
@@ -1115,18 +1125,38 @@ find_trash_in_one_hierarchy_level (const char *current_directory, dev_t near_dev
 
 	item_buffer = g_malloc (sizeof (struct dirent) + GET_PATH_MAX() + 1);
 	for (;;) {
+#ifdef HAVE_READDIR_R
 		if (readdir_r (directory, item_buffer, &item) != 0 || item == NULL) {
 			break;
 		}
-
-		if (gnome_vfs_context_check_cancellation (context))
+#else
+		G_LOCK (readdir);
+		item = readdir (directory);
+		if (item == NULL) {
+			G_UNLOCK (readdir);
 			break;
+		}
+#endif
+
+		if (gnome_vfs_context_check_cancellation (context)) {
+#ifndef HAVE_READDIR_R
+			G_UNLOCK (readdir);
+#endif
+			break;
+		}
 
 		if (strcmp (item->d_name, ".") == 0
-			|| strcmp (item->d_name, "..") == 0)
+			|| strcmp (item->d_name, "..") == 0) {
+#ifndef HAVE_READDIR_R
+			G_UNLOCK (readdir);
+#endif
 			continue;
+		}
 
 		item_path = append_to_path (current_directory, item->d_name);
+#ifndef HAVE_READDIR_R
+		G_UNLOCK (readdir);
+#endif
 		if (lstat (item_path, &stat_buffer) == 0 
 			&& S_ISDIR (stat_buffer.st_mode)
 			&& near_device_id == stat_buffer.st_dev) {
